"""
Get the absolute timing of the pulses
by use of the gps time
Calculate also a non hex representation of
leading and falling edges of the pulses
"""


# for the pulses 
# 8 bits give a hex number
# but only the first 5 bits are used for the pulses time,
# the fifth bit flags if the pulse is considered valid
# the seventh bit should be the triggerflag...

BIT0_4 = 31
BIT5 = 1 << 5
BIT7 = 1 << 7

# For DAQ status
BIT0 = 1 # 1 PPS interrupt pending
BIT1 = 1 << 1 # Trigger interrupt pending
BIT2 = 1 << 2 # GPS data possible corrupted
BIT3 = 1 << 3 # Current or last 1PPS rate not within range

# ticksize of tmc internal clock
# documentation says 0.75, measurement says 1.25
# TODO: find out if tmc is coupled to cpld!
TMC_TICK = 1.25 #nsec
#MAX_TRIGGERWINDOW = 60.0 #nsec
MAX_TRIGGERWINDOW = 9960.0 #nsec for mudecay!
DEFAULT_FREQUENCY = 25.0e6


class PulseExtractor:
    """
    get the pulses out of a daq line
    speed is important here
    """

    def __init__(self,pulsefile=''):
        """
        if a pulsefile is given, all the extracte pulses
        will be written into it
        """
 
        #TODO change to dictionaries, they might allthoug be faster
        
        self.re      = {"ch0" : [], "ch1" : [], "ch2" : [], "ch3" : [] }
        self.fe      = {"ch0" : [], "ch1" : [], "ch2" : [], "ch3" : [] }
        self.last_re = {"ch0" : [], "ch1" : [], "ch2" : [], "ch3" : [] }
        self.last_fe = {"ch0" : [], "ch1" : [], "ch2" : [], "ch3" : [] }

        # ini will be False if we have seen the first 
        # trigger
        # store items if Events are longer than one line
        self.ini                 = True
        self.lastonepps          = 0
        self.lasttriggertime     = 0
        self.trigger_count       = 0
        self.lasttime            = 0


        # store the actual value of
        # the trigger counter
        # to correct trigger counter rollover
        self.lasttriggercount = 0

        # TODO find a generic way to account
        # for the fact that the default
        # cna be either 25 or 41 MHz
        # variables for DAQ frequency calculation
        self.lastfrequencypolltime = 0
        self.lastfrequencypolltriggers = 0
        self.calculatedfrequency = DEFAULT_FREQUENCY
        self.lastoneppspoll      = 0
        self.passedonepps        = 0 
        self.prevlast_onepps     = 0

        #self.debug_freq = open('frequency.txt','w')
        #self.debug_freq.write('#  sec (le (nsec), fe(nsec)) chan0 chan1 chan2 chan3 \n')
        if pulsefile:
            self.pulsefile = open(pulsefile,'w')           
        else:
            self.pulsefile = False

    def _calculate_edges(self,line,counter_diff=0):
        """
        get the leading and falling edges of the pulses
        Use counter diff for getting pulse times in subsequent 
        lines of the triggerflag
        """
        
        rising_edges  = {"ch0" : int(line[1],16), "ch1" : int(line[3],16), "ch2" : int(line[5],16), "ch3" :int(line[7],16)}
        falling_edges  = {"ch0" : int(line[2],16), "ch1" : int(line[4],16), "ch2" : int(line[6],16), "ch3" :int(line[8],16)}


        for ch in ["ch0","ch1","ch2","ch3"]:
            re = rising_edges[ch]
            fe = falling_edges[ch]
            if (re & BIT5):
                self.re[ch].append(counter_diff + (re & BIT0_4)*TMC_TICK ) 
            if (fe & BIT5):
                self.fe[ch].append(counter_diff + (fe & BIT0_4)*TMC_TICK ) 



    def _order_and_cleanpulses(self):
        """
        Remove pulses which have a 
        leading edge later in time than a 
        falling edge and do a bit of sorting
        Remove also single leading or falling edges
        NEW: We add virtual falling edges!
        """
        pulses  = {"ch0" : [], "ch1" : [], "ch2" : [], "ch3" : [] }


        for ch in ["ch0","ch1","ch2","ch3"]:
            
            for index,re in enumerate(self.last_re[ch]):
                # add the virtual falling edge if necessary
                try:
                    fe = self.last_fe[ch][index]
                    if fe < re:
                        fe = MAX_TRIGGERWINDOW 
                except IndexError:
                    fe = MAX_TRIGGERWINDOW
                    
                pulses[ch].append((re,fe))
                
            pulses[ch] = sorted(pulses[ch])

           # self.pulses[ch] = [(re,fe) for re,fe in self.last_re[ch],self.last_fe[ch])
           # for i in self.pulses[ch]:
           #     if not i[0] < i[1]:
           #         #self.chan0.remove(i)
           #         self.pulses[ch] = (i[0],MAX_TRIGGERWINDOW)
        return pulses

    def _get_evt_time(self,time,correction,trigger_count,onepps):
        """
        Get the absolute event time in seconds since day start
        If gps is not available, only relative eventtime based on counts
        is returned
        """
        tfields = time.split(".")
        t = tfields[0]

        secs_since_day_start = int(t[0:2])*3600+int(t[2:4])*60+int(t[4:6])

        # FIXME: Why tfields[1]/1000?
        gps_time = float(secs_since_day_start + int(tfields[1])/1000.0+int(correction)/1000.0)

        line_time = gps_time + float((trigger_count - onepps)/self.calculatedfrequency)
        return line_time


    def extract(self,line):
        """
        Analyze subsequent lines (one per call)
        and check if pulses are related to triggers
        For each new trigger,
        return the set of pulses which belong to that trigger,
        otherwise return None
        """

        line = line.split()

        onepps        = int(line[9],16)
        trigger_count = int(line[0],16)
        time          = line[10]

        # correct for triggercount rollover
        if trigger_count < self.lasttriggercount:
            trigger_count += int(0xFFFFFFFF) # counter offset

        self.trigger_count = trigger_count
        
        switched_onepps = False
        if onepps != self.lastonepps:
            self.passedonepps += 1
            # poll every x lines for the frequency
            # check for onepps counter rollover
            if onepps < self.lastonepps:
                onepps += int(0xFFFFFFFF)
        
            switched_onepps = True
            
            # calculate the frequency every x onepps
            if not self.passedonepps%5:
                self.calculatedfrequency = (onepps - self.lastoneppspoll)/float(self.passedonepps)
                self.passedonepps = 0
                self.lastoneppspoll = onepps
                # check if calculatedfrequency is sane,
                # assuming the daq frequency is somewhat stable
                if not 0.5*self.calculatedfrequency < DEFAULT_FREQUENCY < 1.5*self.calculatedfrequency:
                    self.calculatedfrequency = DEFAULT_FREQUENCY

       
            if time == self.lasttime: 
            # correcting for delayed onepps swicth
                linetime = self._get_evt_time(line[10],line[15],trigger_count,self.lastonepps)
            else:
                linetime = self._get_evt_time(line[10],line[15],trigger_count,onepps)


        else:
            linetime = self._get_evt_time(line[10],line[15],trigger_count,onepps)

        # storing the last two onepps switches
        self.prevlast_onepps = self.lastonepps
        self.lastonepps = onepps

        self.lasttime = time

        if int(line[1],16) & BIT7: # a triggerflag!
            self.ini = False
             
            # a new trigger! we have to evaluate the last one and get the new pulses
        
            self.last_re = self.re
            self.last_fe = self.fe
            pulses = self._order_and_cleanpulses()
            extracted_pulses = (self.lasttriggertime,pulses["ch0"],pulses["ch1"],pulses["ch2"],pulses["ch3"])
            if self.pulsefile:
                self.pulsefile.write(extracted_pulses.__repr__() + '\n')

            # as the pulses for the last event are done, reinitialize data structures
            # for the next event
            self.lasttriggertime = linetime
            self.re = {"ch0" : [], "ch1" : [], "ch2" : [], "ch3" : [] }
            self.fe = {"ch0" : [], "ch1" : [], "ch2" : [], "ch3" : [] }

            # calculate edges of the new pulses
            self._calculate_edges(line)
            self.lasttriggercount = trigger_count 
        
            return extracted_pulses
        
        else:    
            # we do have a previous trigger and are now adding more pulses to the event
            #print 'notrigger',line
        
            if self.ini:
                self.lastonepps = int(line[9],16)
                
            else:
                counter_diff = 0
                counter_diff = (self.trigger_count - self.lasttriggercount)
                #print counter_diff, counter_diff > int(0xffffffff)
                # FIXME: is this correct?
                if counter_diff > int(0xffffffff):
                    counter_diff -= int(0xffffffff)
        
                counter_diff = counter_diff/self.calculatedfrequency
                
                #print counter_diff,counter_diff*1e9,' edges diff'
       
                 
                self._calculate_edges(line,counter_diff=counter_diff*1e9)
        # end of if triggerflag

        self.lasttriggercount = trigger_count 


    def close_file(self):
        self.pulsefile.close()          


#ma a velocity "trigger", so that czts can be defined
class VelocityTrigger:
    
    def __init__(self,logger):
        self.logger = logger
        self.logger.info("Velocity trigger initialized")
        
    def trigger(self,pulses,upperchannel=1,lowerchannel=2):
        """
        Timedifference will be calculated t(upperchannel) - t(lowerchannel)
        """
        # remember that index 0 is the triggertime
        upperpulses = len(pulses[upperchannel])
        lowerpulses = len(pulses[lowerchannel])
        
        if upperpulses and lowerpulses:
            if len(pulses[upperchannel][0]) > 1 and len(pulses[lowerchannel][0]) > 1:
                pulsewidth = []

                pulsewidth.append(pulses[upperchannel][0][1] - pulses[upperchannel][0][0])
                pulsewidth.append(pulses[lowerchannel][0][1] - pulses[lowerchannel][0][0])
                if pulsewidth[0]-pulsewidth[1] < -15. or pulsewidth[0]-pulsewidth[1] > 45.:
                    return None
            else:
                return None
            
            tdiff = pulses[lowerchannel][0][0] - pulses[upperchannel][0][0] # always use rising edge since fe might be virtual
            return tdiff
            


# trigger on a set of extracted pulses and look for decayed muons
class DecayTriggerThorough:
    """
    We demand a second pulse in the same channel where the muon got stuck
    Should operate for a 10mu sec triggerwindow
    """   

    def __init__(self,logger):
        self.triggerwindow = 10000  # 10 musec set at DAQ -> in ns since
                                    # TMC info is in nsec
                                   
        self.logger = logger
        self.logger.info("Initializing decay trigger, setting triggerwindow to %i" %self.triggerwindow)

    def trigger(self,triggerpulses,single_channel = 2, double_channel = 3, veto_channel = 4,mindecaytime=0,minsinglepulsewidth=0,maxsinglepulsewidth=12000,mindoublepulsewidth=0,maxdoublepulsewidth=12000):
        """
        Trigger on a certain combination of single and doublepulses
        """ 
        #print single_channel, double_channel, veto_channel,mindecaytime

        ttp = triggerpulses       
        pulses1 = len(ttp[single_channel]) # single pulse 
        pulses2 = len(ttp[double_channel]) # double pulse
        pulses3 = len(ttp[veto_channel])   # veto pulses
        time    = ttp[0]
        #print ttp
        decaytime = 0

        # reject events with too few pulses
        # in some setups good value will be three
        # (single pulse + double pulse required)
        # and no hits in the veto channel3
        # change this if only one channel is available

        if (pulses1 + pulses2 < 2) or pulses3:
            # reject event if it has to few pulses
            # or veto pulses
            self.logger.debug("Rejecting decay with singlepulses %s, doublepulses %s and vetopulses %s" %(pulses1.__repr__(),pulses2.__repr__(),pulses3.__repr__()))
            return None

        # muon it might have entered the second channel
        # then we do not want to have more than one
        # hit in the first
        # again: use selfveto to adjust the behavior
        if single_channel == double_channel:
            if pulses2 >= 2 and pulses1 >= 2:
                # check if the width of the pulses is as required
                singlepulsewidth = ttp[single_channel][0][1] - ttp[single_channel][0][0]
                doublepulsewidth = ttp[double_channel][-1][1] - ttp[double_channel][-1][0]
                #print singlepulsewidth,minsinglepulsewidth,maxsinglepulsewidth,"single",minsinglepulsewidth < singlepulsewidth < maxsinglepulsewidth
                #print doublepulsewidth,mindoublepulsewidth,maxdoublepulsewidth,"double",mindoublepulsewidth < doublepulsewidth < maxdoublepulsewidth
                if (minsinglepulsewidth < singlepulsewidth < maxsinglepulsewidth)  and (mindoublepulsewidth < doublepulsewidth < maxdoublepulsewidth):          
                    # subtract rising edges, falling edges might be virtual
                    decaytime = ttp[double_channel][-1][0] - ttp[double_channel][0][0]
                else:
                    self.logger.debug('Rejected event.')
                    return None
            else:
                self.logger.debug('Rejected event.')
                return None
        else:
            if pulses2 >= 2 and pulses1 == 1:
                # check if the width of the pulses is as required
                singlepulsewidth = ttp[single_channel][0][1] - ttp[single_channel][0][0]
                doublepulsewidth = ttp[double_channel][-1][1] - ttp[double_channel][-1][0]
                #print singlepulsewidth,minsinglepulsewidth,maxsinglepulsewidth,"single",minsinglepulsewidth < singlepulsewidth < maxsinglepulsewidth
                #print doublepulsewidth,mindoublepulsewidth,maxdoublepulsewidth,"double",mindoublepulsewidth < doublepulsewidth < maxdoublepulsewidth
                if (minsinglepulsewidth < singlepulsewidth < maxsinglepulsewidth)  and (mindoublepulsewidth < doublepulsewidth < maxdoublepulsewidth):          
                    # subtract rising edges, falling edges might be virtual
                    decaytime = ttp[double_channel][-1][0] - ttp[double_channel][0][0]
                else:
                    self.logger.debug('Rejected event.')
                    return None
            else:
                self.logger.debug('Rejected event.')
                return None

        # perform sanity checks
        if (decaytime > mindecaytime) and (decaytime < self.triggerwindow -1000): # there is an artefact at the end
            self.logger.debug("Decay with decaytime %d found " %decaytime)        # of the triggerwindow
            return decaytime                                                      # so -1000
        self.logger.debug("Rejecting decay with singlepulses %s, doublepulses %s and vetopulses %s" %(pulses1.__repr__(),pulses2.__repr__(),pulses3.__repr__()))
        return None


if __name__ == '__main__':

    import sys 

    data = open(sys.argv[1])
    extractor = PulseExtractor()
    while True:
        line = data.readline()
        if not line:
            break
        try:
            extractor.extract(line)
            #print extractor.extract(line)
        except (ValueError,IndexError):
            pass 



"""
Script for performing a fit to a histogramm of recorded 
time differences for the use with QNet
"""


import scipy.optimize as optimize
import numpy
import pylab
import sys
#import optimalbins

def main(bincontent=None,binning = (0,10,21), fitrange = None):

    def decay(p,x):
        return p[0]*numpy.exp(-x/p[1])+p[2]
    
    def error(p,x,y):
        return decay(p,x)-y
    
    if bincontent is None:
    
        nbins = 10
        xmin = 1.0
        xmax = 20.0
    

        times = [float(l) for l in open(sys.argv[1]).readlines() if xmin<float(l)<xmax]
        print len(times),"decay times"

	    #nbins = optimalbins.optbinsize(times,1,80)    
	    #print nbins, 'Optimalbins selects nbins'    
       

        #nbins = optimalbins.optbinsize(times,1,30)
        print "Nbins:",nbins
        
        bin_edges = numpy.linspace(binning[0],binning[1],binning[2])
        bin_centers = bin_edges[:-1] + 0.5*(bin_edges[1]-bin_edges[0])
        
        hist,edges = numpy.histogram(times,bin_edges)
        
        #hist=hist[:-1]
        p0 = numpy.array([200,2.0,5])
        
        output = optimize.leastsq(error,p0,args=(bin_centers,hist),full_output=1)
        p = output[0]
        covar = output[1]
        
        print "Fit parameters:",p
        print "Covariance matrix:",covar
        
        chisquare=0.
        deviations=error(p,bin_centers,hist)
        for i,d in enumerate(deviations):
            chisquare += d*d/decay(p,bin_centers[i])
        
        params = {'legend.fontsize': 13}
        pylab.rcParams.update(params)
        
        fitx=numpy.linspace(xmin,xmax,100)
        pylab.plot(bin_centers,hist,"b^",fitx,decay(p,fitx),"b-")
        pylab.ylim(0,max(hist)+100)
        pylab.xlabel("Decay time in microseconds")
        pylab.ylabel("Events in time bin")
        # pylab.legend(("Data","Fit: (%4.2f +- %4.2f) microsec,chisq/ndf=%4.2f"%(p[1],numpy.sqrt(covar[1][1]),chisquare/(nbins-len(p)))))
        pylab.legend(("Data","Fit: (%4.2f) microsec,chisq/ndf=%4.2f"%(p[1]),chisquare/(nbins-len(p))))

	pylab.grid()
        pylab.savefig("fit.png")
        
    else:

        # this is then used for the mudecaywindow
        # in muonic
        # we have to adjust the bins
        # to the values of the used histogram
        if len(bincontent) == 0:
            print 'WARNING: Empty bins.'
            return None
    
        bins = numpy.linspace(binning[0],binning[1],binning[2])
        bin_centers = bins[:-1] + 0.5*(bins[1]-bins[0])
        if fitrange is not None:
            if fitrange[0] < binning[0]:
                fitrange = (binning[0], fitrange[1])
            if fitrange[1] > binning[1]:
                fitrange = (fitrange[0],binning[1])
            bin_mask = [(bin_centers <= fitrange[1]) & (bin_centers >= fitrange[0])]
            bin_centers_ = numpy.asarray([x for x in bin_centers if (x <= fitrange[1] and x >= fitrange[0])])
            if len(bin_centers_) < 3:
                print 'WARNING: fit range too small. Skipping fitting. Try with larger fit range.'
                return None
            else:
                bin_centers = bin_centers_
                bincontent = bincontent[bin_mask]

        # we cut the leading edge of the distribution away for the fit 
        glob_max = max(bincontent)
        cut = 0
        for i in enumerate(bincontent):
            if i[1] == glob_max:
                cut = i[0]

        cut_bincontent  = bincontent[cut:]        
        cut_bincenter   = bin_centers[cut]
        cut_bincenters  = bin_centers[cut:]


        # maybe something for the future..
        #nbins = optimalbins.optbinsize(cut_bincontent,1,20)       
        #fit_bins        = n.linspace(cut_bincenter,20,nbins)
        #fit_bin_centers = fit_bins[:-1] + 0.5*(fit_bins[1]-fit_bins[0])
        #fit_bincontent  = n.zeros(len(fit_bin_centers))

        ## the bincontent must be redistributed to fit_bincontent

        #for binindex_fit in xrange(len(fit_bincontent)):
        #    for binindex,content in enumerate(bincontent):
        #        if bin_centers[binindex] <= fit_bin_centers[binindex_fit]:
        #            fit_bincontent[binindex_fit] += content

        p0 = numpy.array([200,2.0,5])

        #output = optimize.leastsq(error,p0,args=(fit_bin_centers,fitbincontent),full_output=1)

        output = optimize.leastsq(error,p0,args=(cut_bincenters,cut_bincontent),full_output=1)

        p = output[0]
        covar = output[1]
        
        print "Fit parameters:",p
        print "Covariance matrix:",covar
        
        chisquare=0.
        deviations=error(p,cut_bincenters,cut_bincontent)
        for i,d in enumerate(deviations):
            chisquare += d*d/decay(p,cut_bincenters[i])
        
        params = {'legend.fontsize': 13}
        pylab.rcParams.update(params)
        
        #nbins = 84
        nbins = len(bins)
        xmin = cut_bincenters[0]
        xmax = cut_bincenters[-1]

        fitx=numpy.linspace(xmin,xmax,100)

        #return (bin_centers,bincontent,fitx,decay,p,covar,chisquare,nbins)
        return (cut_bincenters,cut_bincontent,fitx,decay,p,covar,chisquare,nbins)
     

def gaussian_fit(bincontent,binning = (0,2,10), fitrange = None):

    def gauss(p,x):
        return p[0]*(1/((p[1]*numpy.sqrt(2*numpy.pi))))*numpy.exp(-0.5*(((x - p[2])/p[1])**2))
    
    def error(p,x,y):
        return gauss(p,x)-y
    
    if len(bincontent) == 0:
        print 'WARNING: Empty bins.'
        return None
    
    # this is then used for the mudecaywindow
    # in muonic
    # we have to adjust the bins
    # to the values of the used histogram

    bins = numpy.linspace(binning[0],binning[1],binning[2])
    bin_centers = bins[:-1] + 0.5*(bins[1]-bins[0])

    if fitrange is not None:
        if fitrange[0] < binning[0]:
            fitrange = (binning[0], fitrange[1])
        if fitrange[1] > binning[1]:
            fitrange = (fitrange[0],binning[1])
        bin_mask = [(bin_centers <= fitrange[1]) & (bin_centers >= fitrange[0])]
        bin_centers_ = numpy.asarray([x for x in bin_centers if (x <= fitrange[1] and x >= fitrange[0])])
        if len(bin_centers_) < 3:
            print 'WARNING: fit range too small. Skipping fitting. Try with larger fit range.'
            return None
        else:
            bin_centers = bin_centers_
            bincontent = bincontent[bin_mask]


    # we cut the leading edge of the distribution away for the fit
    #glob_max = max(bincontent)
    #cut = 0
    #for i in enumerate(bincontent):
    #    if i[1] == glob_max:
    #        cut = i[0]


    cut_bincontent  = bincontent#[cut:]        
    cut_bincenter   = bin_centers#[cut]
    cut_bincenters  = bin_centers#[cut:]


    # maybe something for the future..
    #nbins = optimalbins.optbinsize(cut_bincontent,1,20)       
    #fit_bins        = n.linspace(cut_bincenter,20,nbins)
    #fit_bin_centers = fit_bins[:-1] + 0.5*(fit_bins[1]-fit_bins[0])
    #fit_bincontent  = n.zeros(len(fit_bin_centers))

    ## the bincontent must be redistributed to fit_bincontent

    #for binindex_fit in xrange(len(fit_bincontent)):
    #    for binindex,content in enumerate(bincontent):
    #        if bin_centers[binindex] <= fit_bin_centers[binindex_fit]:
    #            fit_bincontent[binindex_fit] += content

#    p0 = numpy.array([20,1.0,5])
    wsum = cut_bincontent.sum()
    mean        = (cut_bincontent * cut_bincenters ).sum() / wsum
    meansquared = (cut_bincontent * cut_bincenters**2 ).sum() /wsum
    var = meansquared - mean**2
    p0 = numpy.array([max(cut_bincontent),var,mean])
    #output = optimize.leastsq(error,p0,args=(fit_bin_centers,fitbincontent),full_output=1)
    output = optimize.leastsq(error,p0,args=(cut_bincenters,cut_bincontent),full_output=1)

    p = output[0]
    covar = output[1]
    
    print "Fit parameters:",p
    print "Covariance matrix:",covar
    
    chisquare=0.
    deviations=error(p,cut_bincenters,cut_bincontent)
    for i,d in enumerate(deviations):
        chisquare += d*d/gauss(p,cut_bincenters[i])
    
    params = {'legend.fontsize': 13}
    pylab.rcParams.update(params)
    
    #nbins = 84
    nbins = len(bins)
    xmin = cut_bincenters[0]
    xmax = cut_bincenters[-1]

    fitx=numpy.linspace(xmin,xmax,100)

    #return (bin_centers,bincontent,fitx,decay,p,covar,chisquare,nbins)
    return (cut_bincenters,cut_bincontent,fitx,gauss,p,covar,chisquare,nbins)
     

if __name__ == '__main__':
    main()



from PyQt4 import QtGui
from PyQt4 import QtCore

class LineEdit(QtGui.QLineEdit):

    def __init__(self, *args):
        QtGui.QLineEdit.__init__(self, *args)
        self.history=[]
        self.hist_pointer = 0
        
    def event(self, event):
        if event.type() == QtCore.QEvent.KeyPress:
            if event.key()==QtCore.Qt.Key_Down:
                self.emit(QtCore.SIGNAL("keyDownPressed"))
                if self.hist_pointer < len(self.history)-1:
                    self.hist_pointer += 1
                    self.setText(self.history[self.hist_pointer])
                elif self.hist_pointer == len(self.history)-1:
                    self.setText('')
                    self.hist_pointer += 1
                return True
            if event.key()==QtCore.Qt.Key_Up:
                self.emit(QtCore.SIGNAL("keyUpPressed"))
                if self.hist_pointer > 0:
                    self.hist_pointer -= 1
                    self.setText(self.history[self.hist_pointer])
                return True
            else:
                return QtGui.QLineEdit.event(self, event)
        return QtGui.QLineEdit.event(self, event)

    def add_hist_item(self,item):
        self.history.append(item)
        self.hist_pointer = len(self.history)

# vim: ai ts=4 sts=4 et sw=4



#!/usr/bin/env python

import sys
import gzip

#IMPORTANT
# The order of the szintillators is 0->2->1
######

#files = sys.argv[1]
files = ("../../../CosmicCode/2012-7-13_7-27-29_RAW_HOURS_Current")
BIT0_4 = 31
BIT5 = 1 << 5
BIT7 = 1 << 7

# For DAQ status
BIT0 = 1 # 1 PPS interrupt pending
BIT1 = 1 << 1 # Trigger interrupt pending
BIT2 = 1 << 2 # GPS data possible corrupted
BIT3 = 1 << 3 # Current or last 1PPS rate not within range

def time_to_seconds(time, correction):
    '''
    Convert hhmmss,xxx string int seconds since day start
    '''
    #print time,correction
    tfields = time.split(".")
    t = tfields[0]
    secs_since_day_start = int(t[0:2])*3600+int(t[2:4])*60+int(t[4:6])
    evt_time = secs_since_day_start + int(tfields[1])/1000.0+int(correction)/1000.0
    return round(evt_time)
    counter  = 0.0 
    

for filename in files:
    
    counter  = 0.0 
    muon = {0:False,1:False,2:False,"Time":0.}
    freq = 25e6 # 25 MHz
    last_onepps = 0
    muon_start = 0.
    nmuons = 0
    last_seconds = 0.
    last_time = 0.
    last_triggercount = 0
    switched_onepps = False
    last_onepps = 0
    onepps_count = 0
    re_previous = 0.0
    
    counter_previous = 0.0
    trigger_count_previous = 0.0
    onepps_count_previous = 0.0
    line_current = ""
    if filename.endswith('.gz'):
        f = gzip.open(filename)
    else:
        f = open(sys.argv[1])
       
for line in f:
   
    counter = counter + 1
    if counter == 1000:
        break
    fields = line.rstrip("\n").split(" ")
        # Ignore malformed lines
    if len(fields) != 16:
        continue
        #Ignore everything that is not trigger data
    if len(fields[0]) != 8:
        continue
  
    try:
        int(fields[len(fields)-1])
    except ValueError:
        continue
       
    ch0 = float(sys.argv[2])
    ch1 = float(sys.argv[3])
    ch2 = float(sys.argv[4])
    ch3 = float(sys.argv[5])
        
    trigger_count = int(fields[0],16)
    onepps_count = int(fields[9],16)
   
    re_a = 0
    re_b = 0
 
    if ch0==1 and ch1==2 and ch2==0 and ch3==0:
        re_a  = int(fields[1],16)*1.25
        re_b  = int(fields[3],16)*1.25
        
    if ch0==1 and ch1==0 and ch2==2 and ch3==0:
        re_a  = int(fields[1],16)*1.25
        re_b  = int(fields[5],16)*1.25

    if ch0==1 and ch1==0 and ch2==0 and ch3==2:
        re_a  = int(fields[1],16)*1.25
        re_b  = int(fields[7],16)*1.25

    if ch0==0 and ch1==1 and ch2==2 and ch3==0:
        re_a  = int(fields[3],16)*1.25
        re_b  = int(fields[5],16)*1.25

    if ch0==0 and ch1==1 and ch2==0 and ch3==2:
        re_a  = int(fields[3],16)*1.25
        re_b  = int(fields[7],16)*1.25

    if ch0==0 and ch1==0 and ch2==1 and ch3==2:
        re_a  = int(fields[5],16)*1.25
        re_b  = int(fields[7],16)*1.25

    #Vice versa! Important for offset measurements 

    if ch0==2 and ch1==1 and ch2==0 and ch3==0:
        re_b  = int(fields[1],16)*1.25
        re_a  = int(fields[3],16)*1.25
        
    if ch0==2 and ch1==0 and ch2==1 and ch3==0:
        re_b  = int(fields[1],16)*1.25
        re_a  = int(fields[5],16)*1.25

    if ch0==2 and ch1==0 and ch2==0 and ch3==1:
        re_b  = int(fields[1],16)*1.25
        re_a  = int(fields[7],16)*1.25

    if ch0==0 and ch1==2 and ch2==1 and ch3==0:
        re_b  = int(fields[3],16)*1.25
        re_a  = int(fields[5],16)*1.25

    if ch0==0 and ch1==2 and ch2==0 and ch3==1:
        re_b  = int(fields[3],16)*1.25
        re_a  = int(fields[7],16)*1.25

    if ch0==0 and ch1==0 and ch2==2 and ch3==1:
        re_b  = int(fields[5],16)*1.25
        re_a  = int(fields[7],16)*1.25

    time = fields[10]
    correction = (fields[15])
        
    if re_b > 0.0 and re_a==0 and re_previous>0 and line_current != line:
        upper = float(time_to_seconds(time_previous,correction_previous)+(trigger_count_previous-onepps_count_previous)+re_previous)
        lower = float(time_to_seconds(time,correction)+(trigger_count-onepps_count)+re_b)
        distance = float(sys.argv[6])
        if (lower-upper) < 0.0:
            velocity = distance/(abs(lower-upper)*pow(10,-2))
            if velocity > 0.0:
                print "velocity muon %10.8f meterperseconds"%(velocity,)
        
    if re_a > 0.0 and re_b==0:
        re_previous = re_a
        time_previous = time
        correction_previous = correction
        trigger_count_previous = trigger_count
        onepps_count_previous = onepps_count
        
    if last_onepps != onepps_count:
        if onepps_count > last_onepps:
            freq  = float(onepps_count - last_onepps)
        else:
            freq = float(0xFFFFFFFF + onepps_count - last_onepps)
                #print "Freq:",freq
            prevlast_onepps = last_onepps
            last_onepps = onepps_count
            switched_onepps = True
                # vim: ai ts=4 sts=4 et sw=4
                
    
